@@ -33,24 +33,32 @@
 #'   \code{\link{set_prior}} or related functions and combined using the
 #'   \code{c} method or the \code{+} operator. See also  \code{\link{get_prior}}
 #'   for more help.
-#' @param autocor An optional \code{\link{cor_brms}} object describing the
-#'   correlation structure within the response variable (i.e., the
-#'   'autocorrelation'). See the documentation of \code{\link{cor_brms}} for a
-#'   description of the available correlation structures. Defaults to
+#' @param data2 A named \code{list} of objects containing data, which
+#'   cannot be passed via argument \code{data}. Required for some objects 
+#'   used in autocorrelation structures to specify dependency structures
+#'   as well as for within-group covariance matrices.
+#' @param autocor (Deprecated) An optional \code{\link{cor_brms}} object
+#'   describing the correlation structure within the response variable (i.e.,
+#'   the 'autocorrelation'). See the documentation of \code{\link{cor_brms}} for
+#'   a description of the available correlation structures. Defaults to
 #'   \code{NULL}, corresponding to no correlations. In multivariate models,
 #'   \code{autocor} might also be a list of autocorrelation structures.
+#'   It is now recommend to specify autocorrelation terms directly
+#'   within \code{formula}. See \code{\link{brmsformula}} for more details.
 #' @param sparse (Deprecated) Logical; indicates whether the population-level
 #'   design matrices should be treated as sparse (defaults to \code{FALSE}). For
 #'   design matrices with many zeros, this can considerably reduce required
 #'   memory. Sampling speed is currently not improved or even slightly
 #'   decreased. It is now recommended to use the \code{sparse} argument of
 #'   \code{\link{brmsformula}} and related functions.
-#' @param cov_ranef A list of matrices that are proportional to the (within)
-#'   covariance structure of the group-level effects. The names of the matrices
-#'   should correspond to columns in \code{data} that are used as grouping
-#'   factors. All levels of the grouping factor should appear as rownames of the
-#'   corresponding matrix. This argument can be used, among others to model
-#'   pedigrees and phylogenetic effects. See
+#' @param cov_ranef (Deprecated) A list of matrices that are proportional to the
+#'   (within) covariance structure of the group-level effects. The names of the
+#'   matrices should correspond to columns in \code{data} that are used as
+#'   grouping factors. All levels of the grouping factor should appear as
+#'   rownames of the corresponding matrix. This argument can be used, among
+#'   others to model pedigrees and phylogenetic effects. 
+#'   It is now recommended to specify those matrices in the formula
+#'   interface using the \code{\link{gr}} and related functions. See
 #'   \code{vignette("brms_phylogenetics")} for more details.
 #' @param save_ranef A flag to indicate if group-level effects for each level of
 #'   the grouping factor(s) should be saved (default is \code{TRUE}). Set to
@@ -65,18 +73,19 @@
 #'   in Stan's \code{parameters} block should be saved (default is
 #'   \code{FALSE}). Saving these samples is required in order to apply the
 #'   methods \code{bridge_sampler}, \code{bayes_factor}, and \code{post_prob}.
-#' @param sample_prior Indicate if samples from priors should be drawn 
-#'   additionally to the posterior samples (defaults to \code{"no"}). Among 
-#'   others, these samples can be used to calculate Bayes factors for point 
-#'   hypotheses via \code{\link{hypothesis}}. Please note that improper priors 
-#'   are not sampled, including the default improper priors used by \code{brm}. 
-#'   See \code{\link{set_prior}} on how to set (proper) priors. Please also note
-#'   that prior samples for the overall intercept are not obtained by default for 
-#'   technical reasons. See \code{\link{brmsformula}} how to obtain prior samples 
-#'   for the intercept. If \code{sample_prior} is set to \code{"only"}, samples 
-#'   are drawn solely from the priors ignoring the likelihood, which allows among 
-#'   others to generate samples from the prior predictive distribution. In this 
-#'   case, all parameters must have proper priors.
+#' @param sample_prior Indicate if samples from priors should be drawn
+#'   additionally to the posterior samples. Options are \code{"no"} (the
+#'   default), \code{"yes"}, and \code{"only"}. Among others, these samples can
+#'   be used to calculate Bayes factors for point hypotheses via
+#'   \code{\link{hypothesis}}. Please note that improper priors are not sampled,
+#'   including the default improper priors used by \code{brm}. See
+#'   \code{\link{set_prior}} on how to set (proper) priors. Please also note
+#'   that prior samples for the overall intercept are not obtained by default
+#'   for technical reasons. See \code{\link{brmsformula}} how to obtain prior
+#'   samples for the intercept. If \code{sample_prior} is set to \code{"only"},
+#'   samples are drawn solely from the priors ignoring the likelihood, which
+#'   allows among others to generate samples from the prior predictive
+#'   distribution. In this case, all parameters must have proper priors.
 #' @param knots Optional list containing user specified knot values to be used
 #'   for basis construction of smoothing terms. See
 #'   \code{\link[mgcv:gamm]{gamm}} for more details.
@@ -86,7 +95,7 @@
 #' @param stan_funs (Deprecated) An optional character string containing
 #'   self-defined  \pkg{Stan} functions, which will be included in the functions
 #'   block of the generated \pkg{Stan} code. It is now recommended to use the
-#'   \code{stanvars} argument for this purpose, instead.
+#'   \code{stanvars} argument for this purpose instead.
 #' @param fit An instance of S3 class \code{brmsfit} derived from a previous
 #'   fit; defaults to \code{NA}. If \code{fit} is of class \code{brmsfit}, the
 #'   compiled model associated with the fitted result is re-used and all
@@ -152,13 +161,18 @@
 #'   files won't be overwritten, you have to manually remove the file in order
 #'   to refit and save the model under an existing file name. The file name
 #'   is stored in the \code{brmsfit} object for later usage.
+#' @param empty Logical. If \code{TRUE}, the Stan model is not created
+#'   and compiled and the corresponding \code{'fit'} slot of the \code{brmsfit}
+#'   object will be empty. This is useful if you have estimated a brms-created
+#'   Stan model outside of \pkg{brms} and want to feed it back into the package.
+#' @param rename For internal use only. 
 #' @param stan_model_args A \code{list} of further arguments passed to
 #'   \code{\link[rstan:stan_model]{stan_model}}.
-#' @param save_dso Logical, defaulting to \code{TRUE}, indicating whether the
-#'   dynamic shared object (DSO) compiled from the C++ code for the model will
-#'   be saved or not. If \code{TRUE}, we can draw samples from the same model in
-#'   another \R session using the saved DSO (i.e., without compiling the C++
-#'   code again).
+#' @param save_dso (Deprecated) Logical, defaulting to \code{TRUE}, indicating
+#'   whether the dynamic shared object (DSO) compiled from the C++ code for the
+#'   model will be saved or not. If \code{TRUE}, we can draw samples from the
+#'   same model in another \R session using the saved DSO (i.e., without
+#'   compiling the C++ code again).
 #' @param ... Further arguments passed to Stan that is to
 #'   \code{\link[rstan:sampling]{sampling}} or \code{\link[rstan:vb]{vb}}.
 #' 
@@ -329,6 +343,17 @@
 #' library(future)
 #' plan(multiprocess)
 #' fit7 <- update(fit7, future = TRUE)
+#' 
+#' 
+#' # fit a model manually via rstan
+#' scode <- make_stancode(count ~ Trt, data = epilepsy)
+#' sdata <- make_standata(count ~ Trt, data = epilepsy)
+#' stanfit <- rstan::stan(model_code = scode, data = sdata)
+#' # feed the Stan model back into brms
+#' fit8 <- brm(count ~ Trt, data = epilepsy, empty = TRUE)
+#' fit8$fit <- stanfit
+#' fit8 <- rename_pars(fit8)
+#' summary(fit8)
 #' }
 #'
 #' @import parallel
@@ -337,10 +362,9 @@
 #' @import Rcpp
 #' @export
 brm <- function(formula, data, family = gaussian(), prior = NULL, 
-                autocor = NULL, cov_ranef = NULL, 
-                sample_prior = c("no", "yes", "only"), 
-                sparse = NULL, knots = NULL, stanvars = NULL,
-                stan_funs = NULL, fit = NA, save_ranef = TRUE, 
+                autocor = NULL, data2 = NULL, cov_ranef = NULL, 
+                sample_prior = "no", sparse = NULL, knots = NULL,
+                stanvars = NULL, stan_funs = NULL, fit = NA, save_ranef = TRUE, 
                 save_mevars = FALSE, save_all_pars = FALSE, 
                 inits = "random", chains = 4, iter = 2000, 
                 warmup = floor(iter / 2), thin = 1,
@@ -348,7 +372,8 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
                 algorithm = c("sampling", "meanfield", "fullrank"),
                 future = getOption("future", FALSE), silent = TRUE, 
                 seed = NA, save_model = NULL, stan_model_args = list(),
-                save_dso = TRUE, file = NULL, ...) {
+                save_dso = TRUE, file = NULL, empty = FALSE,
+                rename = TRUE, ...) {
   
   if (!is.null(file)) {
     x <- read_brmsfit(file)
@@ -359,8 +384,6 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
   
   # validate arguments later passed to Stan
   dots <- list(...)
-  testmode <- isTRUE(dots$testmode)
-  dots$testmode <- NULL
   algorithm <- match.arg(algorithm)
   silent <- as_one_logical(silent)
   iter <- as_one_numeric(iter)
@@ -370,6 +393,8 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
   cores <- as_one_numeric(cores)
   future <- as_one_logical(future) && chains > 0L
   seed <- as_one_numeric(seed, allow_na = TRUE)
+  empty <- as_one_logical(empty)
+  rename <- as_one_logical(rename)
   if (is.character(inits) && !inits %in% c("random", "0")) {
     inits <- get(inits, mode = "function", envir = parent.frame())
   }
@@ -377,53 +402,56 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
   if (is.brmsfit(fit)) {
     # re-use existing model
     x <- fit
-    icnames <- c("loo", "waic", "kfold", "R2", "marglik")
-    x[icnames] <- list(NULL)
     sdata <- standata(x)
+    x$criteria <- list()
     x$fit <- rstan::get_stanmodel(x$fit)
   } else {  
     # build new model
     formula <- validate_formula(
       formula, data = data, family = family, 
-      autocor = autocor, sparse = sparse
+      autocor = autocor, sparse = sparse,
+      cov_ranef = cov_ranef
     )
     family <- get_element(formula, "family")
-    autocor <- get_element(formula, "autocor")
-    bterms <- parse_bf(formula)
-    data.name <- substitute_name(data)
-    data <- update_data(data, bterms = bterms)
-    prior <- check_prior(
-      prior, formula = formula, data = data,
-      sample_prior = sample_prior, warn = FALSE
+    bterms <- brmsterms(formula)
+    data_name <- substitute_name(data)
+    data <- validate_data(data, bterms = bterms, knots = knots)
+    attr(data, "data_name") <- data_name
+    data2 <- validate_data2(
+      data2, bterms = bterms, 
+      get_data2_autocor(formula),
+      get_data2_cov_ranef(formula)
     )
+    prior <- validate_prior(
+      prior, bterms = bterms, data = data,
+      sample_prior = sample_prior
+    )
+    stanvars <- validate_stanvars(stanvars, stan_funs = stan_funs)
     # initialize S3 object
     x <- brmsfit(
-      formula = formula, family = family, data = data, 
-      data.name = data.name, prior = prior, 
-      autocor = autocor, cov_ranef = cov_ranef, 
-      stanvars = stanvars, stan_funs = stan_funs,
-      algorithm = algorithm
+      formula = formula, data = data, data2 = data2, prior = prior, 
+      stanvars = stanvars, algorithm = algorithm, family = family
     )
     x$ranef <- tidy_ranef(bterms, data = x$data)  
     x$exclude <- exclude_pars(
-      bterms, data = x$data, ranef = x$ranef, 
-      save_ranef = save_ranef, save_mevars = save_mevars,
+      x, save_ranef = save_ranef, 
+      save_mevars = save_mevars,
       save_all_pars = save_all_pars
     )
-    x$model <- make_stancode(
-      formula, data = data, prior = prior, 
-      cov_ranef = cov_ranef,
-      sample_prior = sample_prior, knots = knots, 
-      stanvars = stanvars, stan_funs = stan_funs, 
-      save_model = save_model
+    x$model <- .make_stancode(
+      bterms, data = data, prior = prior, 
+      stanvars = stanvars, save_model = save_model
     )
     # generate Stan data before compiling the model to avoid
     # unnecessary compilations in case of invalid data
-    sdata <- make_standata(
-      formula, data = data, prior = prior, 
-      cov_ranef = cov_ranef, sample_prior = sample_prior,
-      knots = knots, stanvars = stanvars
+    sdata <- .make_standata(
+      bterms, data = data, prior = prior, 
+      data2 = data2, stanvars = stanvars
     )
+    if (empty) {
+      # return the brmsfit object with an empty 'fit' slot
+      return(x)
+    }
     stopifnot(is.list(stan_model_args))
     silence_stan_model <- !length(stan_model_args)
     stan_model_args$model_code <- x$model
@@ -479,8 +507,8 @@ brm <- function(formula, data, family = gaussian(), prior = NULL,
     # vb does not support parallel execution
     x$fit <- do_call(rstan::vb, args)
   }
-  if (!testmode) {
-    x <- rename_pars(x)
+  if (rename) {
+    x <- rename_pars(x) 
   }
   if (!is.null(file)) {
     write_brmsfit(x, file)
