@@ -15,36 +15,35 @@ data_predictor <- function(x, ...) {
 }
 
 #' @export
-data_predictor.mvbrmsterms <- function(x, data, old_sdata = NULL, ...) {
+data_predictor.mvbrmsterms <- function(x, data, basis = NULL, ...) {
   out <- list(N = nrow(data))
-  for (i in seq_along(x$terms)) {
-    od <- old_sdata[[x$responses[i]]]
-    c(out) <- data_predictor(x$terms[[i]], data = data, old_sdata = od, ...)
+  for (r in names(x$terms)) {
+    bs <- basis$resps[[r]]
+    c(out) <- data_predictor(x$terms[[r]], data = data, basis = bs, ...)
   }
   out
 }
 
 #' @export
-data_predictor.brmsterms <- function(x, data, prior, ranef, knots = NULL, 
-                                     old_sdata = NULL, ...) {
+data_predictor.brmsterms <- function(x, data, prior, ranef, 
+                                     basis = NULL, ...) {
   out <- list()
   data <- subset_data(data, x)
   resp <- usc(combine_prefix(x))
-  args_eff <- nlist(data, ranef, prior, knots)
+  args_eff <- nlist(data, ranef, prior, ...)
   for (dp in names(x$dpars)) {
-    args_eff_spec <- list(x = x$dpars[[dp]], old_sdata = old_sdata[[dp]])
+    args_eff_spec <- list(x = x$dpars[[dp]], basis = basis$dpars[[dp]])
     c(out) <- do_call(data_predictor, c(args_eff_spec, args_eff))
   }
   for (dp in names(x$fdpars)) {
     out[[paste0(dp, resp)]] <- x$fdpars[[dp]]$value
   }
   for (nlp in names(x$nlpars)) {
-    args_eff_spec <- list(x = x$nlpars[[nlp]], old_sdata = old_sdata[[nlp]])
+    args_eff_spec <- list(x = x$nlpars[[nlp]], basis = basis$nlpars[[nlp]])
     c(out) <- do_call(data_predictor, c(args_eff_spec, args_eff))
   }
   c(out) <- data_gr_local(x, data = data, ranef = ranef)
   c(out) <- data_mixture(x, prior = prior)
-  c(out) <- data_autocor(x, data = data, old_locations = old_sdata$locations)
   out
 }
 
@@ -52,41 +51,34 @@ data_predictor.brmsterms <- function(x, data, prior, ranef, knots = NULL,
 # @param data the data passed by the user
 # @param ranef object retuend by 'tidy_ranef'
 # @param prior an object of class brmsprior
-# @param knots optional knot values for smoothing terms
-# @param old_sdata see 'extract_old_standata'
+# @param basis information from original Stan data used to correctly 
+#   predict from new data. See 'standata_basis' for details.
 # @param ... currently ignored
 # @return a named list of data to be passed to Stan
 #' @export
 data_predictor.btl <- function(x, data, ranef = empty_ranef(), 
-                               prior = brmsprior(), knots = NULL, 
-                               old_sdata = NULL, ...) {
-  c(data_fe(x, data, ...),
-    data_sp(x, data, prior = prior, Jmo = old_sdata$Jmo),
+                               prior = brmsprior(), data2 = list(),
+                               basis = NULL, ...) {
+  c(data_fe(x, data),
+    data_sp(x, data, prior = prior, basis = basis$sp),
     data_re(x, data, ranef = ranef),
     data_cs(x, data),
-    data_sm(x, data, knots = knots, smooths = old_sdata$smooths),
-    data_gp(x, data, gps = old_sdata$gps),
+    data_sm(x, data, basis = basis$sm),
+    data_gp(x, data, basis = basis$gp),
+    data_ac(x, data, data2 = data2, basis = basis$ac),
     data_offset(x, data),
-    data_bhaz(x, data, basis = old_sdata$base_basis),
+    data_bhaz(x, data, basis = basis$bhaz),
     data_prior(x, data, prior = prior)
   )
 }
 
 # prepare data for non-linear parameters for use in Stan
 #' @export 
-data_predictor.btnl <- function(x, data, ...) {
+data_predictor.btnl <- function(x, data, data2 = list(), 
+                                basis = NULL, ...) {
   out <- list()
-  C <- get_model_matrix(x$covars, data = data)
-  if (length(all.vars(x$covars)) != NCOL(C)) {
-    stop2("Factors with more than two levels are not allowed as covariates.")
-  }
-  # fixes issue #127 occuring for factorial covariates
-  colnames(C) <- all.vars(x$covars)
-  p <- usc(combine_prefix(x))
-  if (NCOL(C)) {
-    out[[paste0("KC", p)]] <- NCOL(C)
-    out[[paste0("C", p)]] <- C
-  }
+  c(out) <- data_cnl(x, data)
+  c(out) <- data_ac(x, data, data2 = data2, basis = basis$ac)
   out
 }
 
@@ -104,32 +96,35 @@ data_fe <- function(bterms, data) {
 }
 
 # data preparation for splines
-data_sm <- function(bterms, data, knots = NULL, smooths = NULL) {
+data_sm <- function(bterms, data, basis = NULL) {
   out <- list()
   smterms <- all_terms(bterms[["sm"]])
   if (!length(smterms)) {
     return(out)
   }
   p <- usc(combine_prefix(bterms))
-  new <- length(smooths) > 0L
+  new <- length(basis) > 0L
   if (!new) {
-    smooths <- named_list(smterms)
+    knots <- get_knots(data)
+    basis <- named_list(smterms)
     for (i in seq_along(smterms)) {
-      smooths[[i]] <- smoothCon(
+      # the spline penality has changed in 2.8.7 (#646)
+      diagonal.penalty <- !require_old_default("2.8.7")
+      basis[[i]] <- smoothCon(
         eval2(smterms[i]), data = data, 
         knots = knots, absorb.cons = TRUE,
-        diagonal.penalty = TRUE
+        diagonal.penalty = diagonal.penalty
       )
     }
   }
   bylevels <- named_list(smterms)
   ns <- 0
   lXs <- list()
-  for (i in seq_along(smooths)) {
+  for (i in seq_along(basis)) {
     # may contain multiple terms when 'by' is a factor
-    for (j in seq_along(smooths[[i]])) {
+    for (j in seq_along(basis[[i]])) {
       ns <- ns + 1
-      sm <- smooths[[i]][[j]]
+      sm <- basis[[i]][[j]]
       if (length(sm$by.level)) {
         bylevels[[i]][j] <- sm$by.level
       }
@@ -281,25 +276,8 @@ data_gr_local <- function(bterms, data, ranef) {
 }
 
 # prepare global data for each group-level-ID
-# @param internal is the data for use in S3 methods only?
-data_gr_global <- function(ranef, cov_ranef = NULL, internal = FALSE) {
+data_gr_global <- function(ranef, data2) {
   out <- list()
-  if (!is.null(cov_ranef) && !internal) {
-    # check validity of cov_ranef
-    cr_names <- names(cov_ranef)
-    cr_is_named <- length(cr_names) && all(nzchar(cr_names))
-    if (!is.list(cov_ranef) || !cr_is_named) {
-      stop2("'cov_ranef' must be a named list.")
-    }
-    if (any(duplicated(cr_names))) {
-      stop2("Names of 'cov_ranef' must be unique.")
-    }
-    unused_cr_names <- setdiff(cr_names, ranef$group)
-    if (length(unused_cr_names)) {
-      stop2("The following elements of 'cov_ranef' are unused: ",
-            collapse_comma(unused_cr_names))
-    }
-  }
   for (id in unique(ranef$id)) {
     tmp <- list()
     id_ranef <- subset2(ranef, id = id)
@@ -317,29 +295,18 @@ data_gr_global <- function(ranef, cov_ranef = NULL, internal = FALSE) {
       tmp$Nby <- length(bylevels)
       tmp$Jby <- as.array(Jby)
     }
-    # prepare customized covariance matrices
-    if (group %in% names(cov_ranef)) {
-      cov_mat <- as.matrix(cov_ranef[[group]])
-      if (!isSymmetric(unname(cov_mat))) {
-        stop2("Covariance matrix of grouping factor '", 
-              group, "' is not symmetric.")
-      }
+    # prepare within-group covariance matrices
+    cov <- id_ranef$cov[1]
+    if (nzchar(cov)) {
+      # validation is only necessary here for compatibility with 'cov_ranef'
+      cov_mat <- validate_recov_matrix(data2[[cov]])
       found_levels <- rownames(cov_mat)
-      if (is.null(found_levels)) {
-        stop2("Row names are required for covariance matrix of '", group, "'.")
-      }
-      colnames(cov_mat) <- found_levels
       found <- levels %in% found_levels
       if (any(!found)) {
-        stop2("Row names of covariance matrix of '", group, 
+        stop2("Levels of the within-group covariance matrix for '", group, 
               "' do not match names of the grouping levels.")
       }
       cov_mat <- cov_mat[levels, levels, drop = FALSE]
-      evs <- eigen(cov_mat, symmetric = TRUE, only.values = TRUE)$values
-      if (min(evs) <= 0) {
-        stop2("Covariance matrix of grouping factor '", 
-              group, "' is not positive definite.")
-      }
       tmp$Lcov <- t(chol(cov_mat))
     }
     names(tmp) <- paste0(names(tmp), "_", id)
@@ -349,7 +316,7 @@ data_gr_global <- function(ranef, cov_ranef = NULL, internal = FALSE) {
 }
 
 # prepare data for special effects for use in Stan
-data_sp <- function(bterms, data, prior = brmsprior(), Jmo = NULL) {
+data_sp <- function(bterms, data, prior = brmsprior(), basis = NULL) {
   out <- list()
   spef <- tidy_spef(bterms, data)
   if (!nrow(spef)) return(out)
@@ -371,8 +338,10 @@ data_sp <- function(bterms, data, prior = brmsprior(), Jmo = NULL) {
     Xmo <- lapply(unlist(spef$calls_mo), get_mo_values, data = data)
     Xmo_names <- paste0("Xmo", p, "_", seq_along(Xmo))
     c(out) <- setNames(Xmo, Xmo_names)
-    compute_Jmo <- is.null(Jmo)
-    if (!length(Jmo)) {
+    if (!is.null(basis$Jmo)) {
+      # take information from original data
+      Jmo <- basis$Jmo
+    } else {
       Jmo <- as.array(ulapply(Xmo, max))
     }
     out[[paste0("Jmo", p)]] <- Jmo
@@ -465,11 +434,11 @@ data_Xme <- function(meef, data) {
 }
 
 # prepare data for Gaussian process terms
-# @param raw store certain intermediate data for further processing?
+# @param internal store some intermediate data for internal post-processing?
 # @param ... passed to '.data_gp'
-data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
+data_gp <- function(bterms, data, internal = FALSE, basis = NULL, ...) {
   out <- list()
-  raw <- as_one_logical(raw)
+  internal <- as_one_logical(internal)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   gpef <- tidy_gpef(bterms, data)
@@ -485,27 +454,14 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
       stop2("Predictors of Gaussian processes should be numeric vectors.")
     }
     Xgp <- do_call(cbind, Xgp)
-    if (gpef$scale[i]) {
-      # scale predictor for easier specification of priors
-      if (length(gps)) {
-        # scale Xgp based on the original data
-        dmax <- gps[[paste0("dmax", pi)]]
-      } else {
-        dmax <- sqrt(max(diff_quad(Xgp)))
-      }
-      if (raw) {
-        # required for scaling of GPs with new data
-        out[[paste0("dmax", pi)]] <- dmax
-      }
-      Xgp <- Xgp / dmax
-    }
     cmc <- gpef$cmc[i]
+    scale <- gpef$scale[i]
     gr <- gpef$gr[i]
     k <- gpef$k[i]
     c <- gpef$c[[i]]
     if (!isNA(k)) {
       out[[paste0("NBgp", pi)]] <- k ^ D
-      Ks <- as.matrix(do.call(expand.grid, repl(seq_len(k), D)))
+      Ks <- as.matrix(do_call(expand.grid, repl(seq_len(k), D)))
     }
     byvar <- gpef$byvars[[i]]
     byfac <- length(gpef$cons[[i]]) > 0L
@@ -524,8 +480,9 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
         Cgp <- con_mat[, j]
         sfx <- paste0(pi, "_", j)
         tmp <- .data_gp(
-          Xgp, k = k, gr = gr, sfx = sfx, Cgp = Cgp, 
-          c = c, raw = raw, gps = gps, ...
+          Xgp, k = k, gr = gr, sfx = sfx, Cgp = Cgp, c = c, 
+          scale = scale, internal = internal, basis = basis, 
+          ...
         )
         Ngp[[j]] <- attributes(tmp)[["Ngp"]]
         Nsubgp[[j]] <- attributes(tmp)[["Nsubgp"]]
@@ -538,8 +495,9 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
     } else {
       out[[paste0("Kgp", pi)]] <- 1L
       c(out) <- .data_gp(
-        Xgp, k = k, gr = gr, sfx = pi, 
-        c = c, raw = raw, gps = gps, ...
+        Xgp, k = k, gr = gr, sfx = pi, c = c, 
+        scale = scale, internal = internal, basis = basis,
+        ...
       )
       if (bynum) {
         Cgp <- as.numeric(get(byvar, data))
@@ -547,6 +505,12 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
       }
     }
   }
+  if (length(basis)) {
+    # original covariate values are required in new GP prediction
+    Xgp_old <- basis[grepl("^Xgp", names(basis))]
+    names(Xgp_old) <- paste0(names(Xgp_old), "_old")
+    out[names(Xgp_old)] <- Xgp_old
+  }
   out
 }
 
@@ -558,7 +522,7 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
 # @param Cgp optional vector of values belonging to
 #   a certain contrast of a factor 'by' variable
 .data_gp <- function(Xgp, k, gr, sfx, Cgp = NULL, c = NULL, 
-                     raw = FALSE, gps = NULL) {
+                     scale = TRUE, internal = FALSE, basis = NULL) {
   out <- list()
   if (!is.null(Cgp)) {
     Cgp <- unname(Cgp)
@@ -580,26 +544,41 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
     }
     out[[paste0("Jgp", sfx)]] <- as.array(Jgp)
     not_dupl_Jgp <- !duplicated(Jgp)
-    Xgp <-  Xgp[not_dupl_Jgp, , drop = FALSE]
+    Xgp <- Xgp[not_dupl_Jgp, , drop = FALSE]
   }
-  if (length(gps)) {
+  if (scale) {
+    # scale predictor for easier specification of priors
+    if (length(basis)) {
+      # scale Xgp based on the original data
+      dmax <- basis[[paste0("dmax", sfx)]]
+    } else {
+      dmax <- sqrt(max(diff_quad(Xgp)))
+    }
+    if (!isTRUE(dmax > 0)) {
+      stop2("Could not scale GP covariates. Please set 'scale' to FALSE in 'gp'.")
+    }
+    if (internal) {
+      # required for scaling of GPs with new data
+      out[[paste0("dmax", sfx)]] <- dmax
+    }
+    Xgp <- Xgp / dmax
+  }
+  if (length(basis)) {
     # center Xgp based on the original data
-    cmeans <- gps[[paste0("cmeans", sfx)]]
+    cmeans <- basis[[paste0("cmeans", sfx)]]
   } else {
     cmeans <- colMeans(Xgp)
   }
-  if (raw) {
-    out[[paste0("Xgp", sfx)]] <- Xgp
-    # required for centering of GPs with new data
+  if (internal) {
+    # required for centering of approximate GPs with new data
     out[[paste0("cmeans", sfx)]] <- cmeans
-    return(out)
   }
   if (!isNA(k)) {
     # basis function approach requires centered variables
     Xgp <- sweep(Xgp, 2, cmeans)
     D <- NCOL(Xgp)
     L <- choose_L(Xgp, c = c)
-    Ks <- as.matrix(do.call(expand.grid, repl(seq_len(k), D)))
+    Ks <- as.matrix(do_call(expand.grid, repl(seq_len(k), D)))
     XgpL <- matrix(nrow = NROW(Xgp), ncol = NROW(Ks))
     slambda <- matrix(nrow = NROW(Ks), ncol = D)
     for (m in seq_rows(Ks)) {
@@ -614,6 +593,157 @@ data_gp <- function(bterms, data, raw = FALSE, gps = NULL, ...) {
   out
 }
 
+# data for autocorrelation variables
+# @param locations optional original locations for CAR models
+data_ac <- function(bterms, data, data2, basis = NULL, ...) {
+  out <- list()
+  N <- nrow(data)
+  acef <- tidy_acef(bterms)
+  if (has_ac_subset(bterms, dim = "time")) {
+    gr <- subset2(acef, dim = "time")$gr
+    if (gr != "NA") {
+      tgroup <- as.numeric(factor(data[[gr]]))
+    } else {
+      tgroup <- rep(1, N) 
+    }
+  }
+  if (has_ac_class(acef, "arma")) {
+    # ARMA correlations
+    acef_arma <- subset2(acef, class = "arma")
+    out$Kar <- acef_arma$p
+    out$Kma <- acef_arma$q
+    if (!use_ac_cov_time(acef_arma)) {
+      # data for the 'predictor' version of ARMA
+      max_lag <- max(out$Kar, out$Kma)
+      out$J_lag <- as.array(rep(0, N))
+      for (n in seq_len(N)[-N]) {
+        ind <- n:max(1, n + 1 - max_lag)
+        # indexes errors to be used in the n+1th prediction
+        out$J_lag[n] <- sum(tgroup[ind] %in% tgroup[n + 1])
+      }
+    }
+  }
+  if (use_ac_cov_time(acef)) {
+    # data for the 'covariance' versions of time-series structures
+    out$N_tg <- length(unique(tgroup))
+    out$begin_tg <- as.array(ulapply(unique(tgroup), match, tgroup))
+    out$nobs_tg <- as.array(with(out, 
+      c(if (N_tg > 1L) begin_tg[2:N_tg], N + 1) - begin_tg
+    ))
+    out$end_tg <- with(out, begin_tg + nobs_tg - 1)
+  }
+  if (has_ac_class(acef, "sar")) {
+    acef_sar <- subset2(acef, class = "sar")
+    M <- data2[[acef_sar$M]]
+    rmd_rows <- attr(data, "na.action")
+    if (!is.null(rmd_rows)) {
+      class(rmd_rows) <- NULL
+      M <- M[-rmd_rows, -rmd_rows, drop = FALSE]
+    }
+    if (!is_equal(dim(M), rep(N, 2))) {
+      stop2("Dimensions of 'M' for SAR terms must be equal to ", 
+            "the number of observations.")
+    }
+    out$Msar <- as.matrix(M)
+    out$eigenMsar <- eigen(M)$values
+    # simplifies code of choose_N
+    out$N_tg <- 1
+  }
+  if (has_ac_class(acef, "car")) {
+    acef_car <- subset2(acef, class = "car")
+    locations <- NULL
+    if (length(basis)) {
+      locations <- basis$locations 
+    }
+    M <- data2[[acef_car$M]]
+    if (acef_car$gr != "NA") {
+      loc_data <- get(acef_car$gr, data)
+      new_locations <- levels(factor(loc_data))
+      if (is.null(locations)) {
+        locations <- new_locations
+      } else {
+        invalid_locations <- setdiff(new_locations, locations)
+        if (length(invalid_locations)) {
+          stop2("Cannot handle new locations in CAR models.")
+        }
+      }
+      Nloc <- length(locations)
+      Jloc <- as.array(match(loc_data, locations))
+      if (is.null(rownames(M))) {
+        stop2("Row names are required for 'M' in CAR terms.")
+      }
+      found <- locations %in% rownames(M)
+      if (any(!found)) {
+        stop2("Row names of 'M' for CAR terms do not match ", 
+              "the names of the grouping levels.")
+      }
+      M <- M[locations, locations, drop = FALSE]
+    } else {
+      warning2(
+        "Using CAR terms without a grouping factor is deprecated. ",
+        "Please use argument 'gr' even if each observation ",
+        "represents its own location."
+      )
+      Nloc <- N
+      Jloc <- as.array(seq_len(Nloc))
+      if (!is_equal(dim(M), rep(Nloc, 2))) {
+        if (length(basis)) {
+          stop2("Cannot handle new data in CAR terms ",
+                "without a grouping factor.")
+        } else {
+          stop2("Dimensions of 'M' for CAR terms must be equal ", 
+                "to the number of observations.")
+        }
+      }
+    }
+    M_tmp <- M
+    M_tmp[upper.tri(M_tmp)] <- NA
+    edges <- which(as.matrix(M_tmp == 1), arr.ind = TRUE)
+    c(out) <- nlist(
+      Nloc, Jloc, Nedges = nrow(edges),  
+      edges1 = as.array(edges[, 1]), 
+      edges2 = as.array(edges[, 2])
+    )
+    if (acef_car$type %in% c("escar", "esicar")) {
+      Nneigh <- Matrix::colSums(M)
+      if (any(Nneigh == 0) && !length(basis)) {
+        stop2(
+          "For exact sparse CAR, all locations should have at ", 
+          "least one neighbor within the provided data set. ",
+          "Consider using type = 'icar' instead."
+        )
+      }
+      inv_sqrt_D <- diag(1 / sqrt(Nneigh))
+      eigenMcar <- t(inv_sqrt_D) %*% M %*% inv_sqrt_D
+      eigenMcar <- eigen(eigenMcar, TRUE, only.values = TRUE)$values
+      c(out) <- nlist(Nneigh, eigenMcar)
+    } else if (acef_car$type %in% "bym2") {
+      c(out) <- list(car_scale = .car_scale(edges, Nloc))
+    }
+  }
+  if (has_ac_class(acef, "fcor")) {
+    acef_fcor <- subset2(acef, class = "fcor")
+    M <- data2[[acef_fcor$M]]
+    rmd_rows <- attr(data, "na.action")
+    if (!is.null(rmd_rows)) {
+      class(rmd_rows) <- NULL
+      M <- M[-rmd_rows, -rmd_rows, drop = FALSE]
+    }
+    if (nrow(M) != N) {
+      stop2("Dimensions of 'M' for FCOR terms must be equal ", 
+            "to the number of observations.") 
+    }
+    out$Mfcor <- M
+    # simplifies code of choose_N
+    out$N_tg <- 1
+  }
+  if (length(out)) {
+    resp <- usc(combine_prefix(bterms))
+    out <- setNames(out, paste0(names(out), resp))
+  }
+  out
+}
+
 # prepare data of offsets for use in Stan
 data_offset <- function(bterms, data) {
   out <- list()
@@ -626,141 +756,35 @@ data_offset <- function(bterms, data) {
     if (length(offset) == 1L) {
       offset <- rep(offset, nrow(data))
     }
-    out[[paste0("offset", p)]] <- as.array(offset)
+    # use 'offsets' as 'offset' will be reserved in stanc3
+    out[[paste0("offsets", p)]] <- as.array(offset)
   }
   out
 }
 
-# data for autocorrelation variables
-# @param old_locations: optional old locations for CAR models
-data_autocor <- function(bterms, data, old_locations = NULL) {
-  stopifnot(is.brmsterms(bterms))
-  autocor <- bterms$autocor
-  N <- nrow(data)
+# data for covariates in non-linear models
+# @param x a btnl object
+# @return a named list of data passed to Stan
+data_cnl <- function(bterms, data) {
+  stopifnot(is.btnl(bterms))
   out <- list()
-  if (is.cor_arma(autocor) || is.cor_cosy(autocor)) {
-    if (length(bterms$time$group)) {
-      tgroup <- as.numeric(factor(data[[bterms$time$group]]))
-    } else {
-      tgroup <- rep(1, N) 
-    }
+  covars <- all.vars(bterms$covars)
+  if (!length(covars)) {
+    return(out)
   }
-  if (is.cor_arma(autocor)) {
-    # ARMA correlations
-    out$Kar <- get_ar(autocor)
-    out$Kma <- get_ma(autocor)
-    if (!use_cov(autocor)) {
-      # data for the 'predictor' version of ARMA
-      max_lag <- max(out$Kar, out$Kma)
-      out$J_lag <- as.array(rep(0, N))
-      for (n in seq_len(N)[-N]) {
-        ind <- n:max(1, n + 1 - max_lag)
-        # indexes errors to be used in the n+1th prediction
-        out$J_lag[n] <- sum(tgroup[ind] %in% tgroup[n + 1])
+  p <- usc(combine_prefix(bterms))
+  for (i in seq_along(covars)) {
+    cvalues <- get(covars[i], data)
+    if (is_like_factor(cvalues)) {
+      # need to apply factor contrasts
+      cform <- str2formula(covars[i])
+      cvalues <- get_model_matrix(cform, data, cols2remove = "(Intercept)")
+      if (NCOL(cvalues) > 1) {
+        stop2("Factors with more than two levels are not allowed as covariates.")
       }
+      cvalues <- cvalues[, 1]
     }
-  }
-  if (use_cov(autocor)) {
-    # data for the 'covariance' versions of ARMA and COSY structures
-    out$N_tg <- length(unique(tgroup))
-    out$begin_tg <- as.array(ulapply(unique(tgroup), match, tgroup))
-    out$nobs_tg <- as.array(with(out, 
-      c(if (N_tg > 1L) begin_tg[2:N_tg], N + 1) - begin_tg
-    ))
-    out$end_tg <- with(out, begin_tg + nobs_tg - 1)
-  }
-  if (is.cor_sar(autocor)) {
-    if (!identical(dim(autocor$W), rep(N, 2))) {
-      stop2("Dimensions of 'W' must be equal to the number of observations.")
-    }
-    out$W <- autocor$W
-    out$eigenW <- eigen(out$W)$values
-    # simplifies code of choose_N
-    out$N_tg <- 1
-  }
-  if (is.cor_car(autocor)) {
-    new <- !is.null(old_locations)
-    if (isTRUE(nzchar(bterms$time$group))) {
-      loc_data <- get(bterms$time$group, data)
-      locations <- levels(factor(loc_data))
-      if (new) {
-        new_locations <- setdiff(locations, old_locations)
-        if (length(new_locations)) {
-          stop2("Cannot handle new locations in CAR models.")
-        }
-      } else {
-        old_locations <- locations
-      }
-      Nloc <- length(locations)
-      Jloc <- as.array(match(loc_data, old_locations))
-      found_locations <- rownames(autocor$W)
-      if (is.null(found_locations)) {
-        stop2("Row names are required for 'W'.")
-      }
-      colnames(autocor$W) <- found_locations
-      found <- locations %in% found_locations
-      if (any(!found)) {
-        stop2("Row names of 'W' do not match ", 
-              "the names of the grouping levels.")
-      }
-      autocor$W <- autocor$W[locations, locations, drop = FALSE]
-    } else {
-      Nloc <- N
-      Jloc <- as.array(seq_len(Nloc))
-      if (!identical(dim(autocor$W), rep(Nloc, 2))) {
-        if (new) {
-          stop2("Cannot handle new data in CAR models ",
-                "without a grouping factor.")
-        } else {
-          stop2("Dimensions of 'W' must be equal ", 
-                "to the number of observations.") 
-        }
-      }
-    }
-    W_tmp <- autocor$W
-    W_tmp[upper.tri(W_tmp)] <- NA
-    edges <- which(as.matrix(W_tmp == 1), arr.ind = TRUE)
-    c(out) <- nlist(
-      Nloc, Jloc, Nedges = nrow(edges),  
-      edges1 = as.array(edges[, 1]), 
-      edges2 = as.array(edges[, 2])
-    )
-    if (autocor$type %in% c("escar", "esicar")) {
-      Nneigh <- Matrix::colSums(autocor$W)
-      if (any(Nneigh == 0)) {
-        stop2(
-          "For exact sparse CAR, all locations should have at ", 
-          "least one neighbor within the provided data set. ",
-          "Consider using type = 'icar' instead."
-        )
-      }
-      inv_sqrt_D <- diag(1 / sqrt(Nneigh))
-      eigenW <- t(inv_sqrt_D) %*% autocor$W %*% inv_sqrt_D
-      eigenW <- eigen(eigenW, TRUE, only.values = TRUE)$values
-      c(out) <- nlist(Nneigh, eigenW)
-    } else if (autocor$type %in% "bym2") {
-      c(out) <- list(car_scale = .car_scale(edges, Nloc))
-    }
-  }
-  if (is.cor_fixed(autocor)) {
-    V <- autocor$V
-    rmd_rows <- attr(data, "na.action")
-    if (!is.null(rmd_rows)) {
-      V <- V[-rmd_rows, -rmd_rows, drop = FALSE]
-    }
-    if (nrow(V) != N) {
-      stop2("'V' must have the same number of rows as 'data'.")
-    }
-    if (min(eigen(V)$values <= 0)) {
-      stop2("'V' must be positive definite.")
-    }
-    out$V <- V
-    # simplifies code of choose_N
-    out$N_tg <- 1
-  }
-  if (length(out)) {
-    resp <- usc(combine_prefix(bterms))
-    out <- setNames(out, paste0(names(out), resp))
+    out[[paste0("C", p, "_", i)]] <- cvalues
   }
   out
 }
@@ -833,7 +857,7 @@ data_prior <- function(bterms, data, prior) {
 # @param cols2remove names of the columns to remove from 
 #   the model matrix; mainly used for intercepts
 # @param rename rename column names via rename()?
-# @param ... currently ignored
+# @param ... passed to stats::model.matrix
 # @return
 #   The design matrix for the given formula and data.
 #   For details see ?stats::model.matrix
@@ -847,7 +871,7 @@ get_model_matrix <- function(formula, data = environment(formula),
   if (no_int(terms)) {
     cols2remove <- union(cols2remove, "(Intercept)")
   }
-  X <- stats::model.matrix(terms, data)
+  X <- stats::model.matrix(terms, data, ...)
   cols2remove <- which(colnames(X) %in% cols2remove)
   if (length(cols2remove)) {
     X <- X[, -cols2remove, drop = FALSE]
