@@ -9,7 +9,7 @@
 #' @param x A \code{brmsfit} object.
 #' @param ... More \code{brmsfit} objects or further arguments
 #'   passed to the underlying post-processing functions.
-#'   In particular, see \code{\link{extract_draws}} for further
+#'   In particular, see \code{\link{prepare_predictions}} for further
 #'   supported arguments.
 #' @param compare A flag indicating if the information criteria
 #'  of the models should be compared to each other
@@ -200,7 +200,7 @@ loo_subsample.brmsfit <- function(x, ..., compare = TRUE, resp = NULL,
   args <- split_dots(x, ..., model_names = model_names)
   c(args) <- nlist(
     criterion = "loo_subsample", compare, resp, 
-    pointwise = TRUE, add_point_draws = TRUE
+    pointwise = TRUE, add_point_estimate = TRUE
   )
   do_call(compute_loos, args)
 }
@@ -717,13 +717,14 @@ reloo.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   .reloo <- function(j) {
     omitted <- obs[j]
     mf_omitted <- mf[-omitted, , drop = FALSE]
-    fit_j <- subset_autocor(x, -omitted, incl_car = TRUE)
+    fit_j <- x
     up_args$object <- fit_j
     up_args$newdata <- mf_omitted
+    up_args$data2 <- subset_data2(x$data2, -omitted)
     fit_j <- SW(do_call(update, up_args))
-    fit_j <- subset_autocor(fit_j, omitted, autocor = x$autocor)
     ll_args$object <- fit_j
     ll_args$newdata <- mf[omitted, , drop = FALSE]
+    ll_args$newdata2 <- subset_data2(x$data2, omitted)
     return(do_call(log_lik, ll_args))
   }
   
@@ -747,8 +748,9 @@ reloo.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   elpd_loo <- ulapply(lls, log_mean_exp)
   # compute \hat{lpd}_j for each of the held out observations (using log-lik
   # matrix from full posterior, not the leave-one-out posteriors)
-  x <- subset_autocor(x, obs)
-  ll_x <- log_lik(x, newdata = mf[obs, , drop = FALSE])
+  mf_obs <- mf[obs, , drop = FALSE]
+  data2_obs <- subset_data2(x$data2, obs)
+  ll_x <- log_lik(x, newdata = mf_obs, newdata2 = data2_obs)
   hat_lpd <- apply(ll_x, 2, log_mean_exp)
   # compute effective number of parameters
   p_loo <- hat_lpd - elpd_loo
@@ -840,19 +842,19 @@ r_eff_log_lik <- function(log_lik, fit, allow_na = FALSE) {
 # methods required in loo_subsample
 #' @importFrom loo .ndraws
 #' @export
-.ndraws.brmsdraws <- function(x, ...) {
+.ndraws.brmsprep <- function(x) {
   x$nsamples
 }
 
 #' @export
-.ndraws.mvbrmsdraws <- function(x, ...) {
+.ndraws.mvbrmsprep <- function(x) {
   x$nsamples
 }
 
 #' @importFrom loo .thin_draws
 #' @export
-.thin_draws.brmsdraws <- function(draws, loo_approximation_draws) {
-  # brmsdraws objects are too complex to implement a post-hoc subsetting method
+.thin_draws.brmsprep <- function(draws, loo_approximation_draws) {
+  # brmsprep objects are too complex to implement a post-hoc subsetting method
   if (length(loo_approximation_draws)) {
     stop2("'loo_approximation_draws' is not supported for brmsfit objects.")
   }
@@ -860,7 +862,7 @@ r_eff_log_lik <- function(log_lik, fit, allow_na = FALSE) {
 }
 
 #' @export
-.thin_draws.mvbrmsdraws <- function(draws, loo_approximation_draws) {
+.thin_draws.mvbrmsprep <- function(draws, loo_approximation_draws) {
   if (length(loo_approximation_draws)) {
     stop2("'loo_approximation_draws' is not supported for brmsfit objects.")
   }
@@ -869,15 +871,15 @@ r_eff_log_lik <- function(log_lik, fit, allow_na = FALSE) {
 
 #' @importFrom loo .compute_point_estimate
 #' @export
-.compute_point_estimate.brmsdraws <- function(x, ...) {
+.compute_point_estimate.brmsprep <- function(draws) {
   # point estimates are stored in the form of an attribute rather 
-  # than computed on the fly due to the complexity of brmsdraws objects
-  attr(x, "point_draws")
+  # than computed on the fly due to the complexity of brmsprep objects
+  attr(draws, "point_estimate")
 }
 
 #' @export
-.compute_point_estimate.mvbrmsdraws <- function(x, ...) {
-  attr(x, "point_draws")
+.compute_point_estimate.mvbrmsprep <- function(draws) {
+  attr(draws, "point_estimate")
 }
 
 # print the output of a list of loo objects
