@@ -138,8 +138,8 @@ stan_llh_weights <- function(llh, bterms, data, resp = "", ...) {
   lpdf <- stan_llh_lpdf_name(bterms)
   Y <- stan_llh_Y_name(bterms)
   glue(
-    "{tp()}weights{resp}[n] * {llh$dist}_{lpdf}", 
-    "({Y}{resp}[n]{llh$shift} | {llh$args}){tr};\n"
+    "{tp()}weights{resp}[n] * ({llh$dist}_{lpdf}", 
+    "({Y}{resp}[n]{llh$shift} | {llh$args}){tr});\n"
   )
 }
 
@@ -306,7 +306,7 @@ stan_llh_gaussian_mv <- function(bterms, resp = "", mix = "") {
   sdist("multi_normal_cholesky", p$Mu, p$LSigma)
 }
 
-stan_llh_gaussian_cov <- function(bterms, resp = "", mix = "") {
+stan_llh_gaussian_time <- function(bterms, resp = "", mix = "") {
   if (stan_llh_adj(bterms)) {
     stop2("Invalid addition arguments for this model.")
   }
@@ -314,36 +314,37 @@ stan_llh_gaussian_cov <- function(bterms, resp = "", mix = "") {
   v <- c("chol_cor", "se2", "nobs_tg", "begin_tg", "end_tg")
   p[v] <- as.list(paste0(v, resp))
   sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
-  sdist(glue("normal_cov_{sfx}"), 
+  sdist(glue("normal_time_{sfx}"), 
     p$mu, p$sigma, p$chol_cor, p$se2,
     p$nobs_tg, p$begin_tg, p$end_tg
   )
 }
 
-stan_llh_gaussian_fixed <- function(bterms, resp = "", mix = "") {
+stan_llh_gaussian_fcor <- function(bterms, resp = "", mix = "") {
   has_se <- is.formula(bterms$adforms$se)
   if (stan_llh_adj(bterms) || has_se) {
     stop2("Invalid addition arguments for this model.")
   }
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
-  p$LV <- paste0("LV", resp)
-  sdist("multi_normal_cholesky", p$mu, p$LV)
+  p$Lfcor <- paste0("Lfcor", resp)
+  sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
+  sdist(glue("normal_fcor_{sfx}"), p$mu, p$sigma, p$Lfcor)
 }
 
 stan_llh_gaussian_lagsar <- function(bterms, resp = "", mix = "") {
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
   p$sigma <- stan_llh_add_se(p$sigma, bterms, FALSE, resp)
-  v <- c("lagsar", "W", "eigenW")
+  v <- c("lagsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("normal_lagsar", p$mu, p$sigma, p$lagsar, p$W, p$eigenW)
+  sdist("normal_lagsar", p$mu, p$sigma, p$lagsar, p$Msar, p$eigenMsar)
 }
 
 stan_llh_gaussian_errorsar <- function(bterms, resp = "", mix = "") {
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
   p$sigma <- stan_llh_add_se(p$sigma, bterms, FALSE, resp)
-  v <- c("errorsar", "W", "eigenW")
+  v <- c("errorsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("normal_errorsar", p$mu, p$sigma, p$errorsar, p$W, p$eigenW)
+  sdist("normal_errorsar", p$mu, p$sigma, p$errorsar, p$Msar, p$eigenMsar)
 }
 
 stan_llh_student <- function(bterms, resp = "", mix = "") {
@@ -361,7 +362,7 @@ stan_llh_student_mv <- function(bterms, resp = "", mix = "") {
   sdist("multi_student_t", p$nu, p$Mu, p$Sigma)
 }
 
-stan_llh_student_cov <- function(bterms, resp = "", mix = "") {
+stan_llh_student_time <- function(bterms, resp = "", mix = "") {
   if (stan_llh_adj(bterms)) {
     stop2("Invalid addition arguments for this model.")
   }
@@ -369,36 +370,39 @@ stan_llh_student_cov <- function(bterms, resp = "", mix = "") {
   v <- c("chol_cor", "se2", "nobs_tg", "begin_tg", "end_tg")
   p[v] <- as.list(paste0(v, resp))
   sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
-  sdist(glue("student_t_cov_{sfx}"), 
+  sdist(glue("student_t_time_{sfx}"), 
     p$nu, p$mu, p$sigma, p$chol_cor, p$se2,
     p$nobs_tg, p$begin_tg, p$end_tg
   )
 }
 
-stan_llh_student_fixed <- function(bterms, resp = "", mix = "") {
+stan_llh_student_fcor <- function(bterms, resp = "", mix = "") {
   has_se <- is.formula(bterms$adforms$se)
   if (stan_llh_adj(bterms) || has_se) {
     stop2("Invalid addition arguments for this model.")
   }
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
-  p$V <- paste0("V", resp)
-  sdist("multi_student_t", p$nu, p$mu, p$V)
+  p$Lfcor <- paste0("Lfcor", resp)
+  sfx <- str_if("sigma" %in% names(bterms$dpars), "het", "hom")
+  sdist(glue("student_t_fcor_{sfx}"), p$nu, p$mu, p$sigma, p$Lfcor)
 }
 
 stan_llh_student_lagsar <- function(bterms, resp = "", mix = "") {
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
   p$sigma <- stan_llh_add_se(p$sigma, bterms, FALSE, resp)
-  v <- c("lagsar", "W", "eigenW")
+  v <- c("lagsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("student_t_lagsar", p$nu, p$mu, p$sigma, p$lagsar, p$W, p$eigenW)
+  sdist("student_t_lagsar", p$nu, p$mu, p$sigma, 
+        p$lagsar, p$Msar, p$eigenMsar)
 }
 
 stan_llh_student_errorsar <- function(bterms, resp = "", mix = "") {
   p <- stan_llh_dpars(bterms, FALSE, resp, mix)
   p$sigma <- stan_llh_add_se(p$sigma, bterms, FALSE, resp)
-  v <- c("errorsar", "W", "eigenW")
+  v <- c("errorsar", "Msar", "eigenMsar")
   p[v] <- as.list(paste0(v, resp))
-  sdist("student_t_errorsar", p$nu, p$mu, p$sigma, p$errorsar, p$W, p$eigenW)
+  sdist("student_t_errorsar", p$nu, p$mu, p$sigma, 
+        p$errorsar, p$Msar, p$eigenMsar)
 }
 
 stan_llh_lognormal <- function(bterms, resp = "", mix = "") {
@@ -534,7 +538,8 @@ stan_llh_exgaussian <- function(bterms, resp = "", mix = "") {
 }
 
 stan_llh_inverse.gaussian <- function(bterms, resp = "", mix = "") {
-  reqn <- stan_llh_adj(bterms) || nzchar(mix)
+  reqn <- stan_llh_adj(bterms) || nzchar(mix) ||
+    glue("shape{mix}") %in% names(bterms$dpars)
   p <- stan_llh_dpars(bterms, reqn, resp, mix)
   lpdf <- paste0("inv_gaussian", if (!reqn) "_vector")
   n <- str_if(reqn, "[n]")
@@ -577,18 +582,7 @@ stan_llh_cox <- function(bterms, resp = "", mix = "") {
 }
 
 stan_llh_cumulative <- function(bterms, resp = "", mix = "") {
-  simplify <- bterms$family$link == "logit" && 
-    !"disc" %in% names(bterms$dpars) && 
-    !has_cs(bterms) && !has_thres_groups(bterms)
-  if (simplify) {
-    prefix <- paste0(resp, if (nzchar(mix)) paste0("_mu", mix))
-    p <- stan_llh_dpars(bterms, TRUE, resp, mix)
-    p$thres <- paste0("Intercept", prefix)
-    out <- sdist("ordered_logistic", p$mu, p$thres)
-  } else {
-    out <- stan_llh_ordinal(bterms, resp, mix)
-  }
-  out
+  stan_llh_ordinal(bterms, resp, mix)
 }
 
 stan_llh_sratio <- function(bterms, resp = "", mix = "") {
@@ -629,7 +623,12 @@ stan_llh_dirichlet <- function(bterms, resp = "", mix = "") {
 stan_llh_ordinal <- function(bterms, resp = "", mix = "") {
   prefix <- paste0(str_if(nzchar(mix), paste0("_mu", mix)), resp)
   p <- stan_llh_dpars(bterms, TRUE, resp, mix)
-  lpdf <- paste0(bterms$family$family, "_", bterms$family$link)
+  if (use_ordered_logistic(bterms)) {
+    lpdf <- "ordered_logistic"
+    p[grepl("^disc", names(p))] <- NULL
+  } else {
+    lpdf <- paste0(bterms$family$family, "_", bterms$family$link) 
+  }
   if (has_thres_groups(bterms)) {
     str_add(lpdf) <- "_merged"
     p$Jthres <- paste0("Jthres", resp, "[n]")
@@ -637,7 +636,10 @@ stan_llh_ordinal <- function(bterms, resp = "", mix = "") {
   } else {
     p$thres <- "Intercept"
   }
-  p$thres <- paste0(p$thres, prefix)
+  str_add(p$thres) <- prefix
+  if (has_sum_to_zero_thres(bterms)) {
+    str_add(p$thres) <- "_stz"
+  }
   if (has_cs(bterms)) {
     if (has_thres_groups(bterms)) {
       stop2("Cannot use category specific effects ", 
@@ -726,7 +728,15 @@ stan_llh_custom <- function(bterms, resp = "", mix = "") {
     prefix <- paste0(resp, if (nzchar(mix)) paste0("_mu", mix))
     p$thres <- paste0("Intercept", prefix)
   }
-  sdist(family$name, p[dpars], p$thres, family$vars)
+  # insert the response name into the 'vars' strings
+  # addition terms contain the response in their variable name
+  var_names <- sub("\\[.+$", "", family$vars)
+  var_indices <- sub("^.+(?=\\[)", "", family$vars, perl = TRUE)
+  is_var_adterms <- var_names %in% c("se", "trials", "dec") |
+    grepl("^((vint)|(vreal))[[:digit:]]+$", var_names)
+  var_resps <- ifelse(is_var_adterms, resp, "")
+  vars <- paste0(var_names, var_resps, var_indices)
+  sdist(family$name, p[dpars], p$thres, vars)
 }
 
 # use Stan GLM primitive functions?
@@ -739,7 +749,7 @@ use_glm_primitive <- function(bterms) {
   mu <- bterms$dpars[["mu"]]
   if (!is.btl(mu) || length(bterms$dpars) > 1L ||
       isTRUE(bterms$rescor) || length(bterms$adforms) ||
-      !is.cor_empty(bterms$autocor)) {
+      is.formula(mu$ac)) {
     return(FALSE)
   }
   # supported families and link functions
@@ -782,6 +792,15 @@ args_glm_primitive <- function(bterms, resp = "") {
   )
 }
 
+# use the ordered_logistic built-in functions
+use_ordered_logistic <- function(bterms) {
+  stopifnot(is.brmsterms(bterms))
+  isTRUE(bterms$family$family == "cumulative") &&
+    isTRUE(bterms$family$link == "logit") && 
+    isTRUE(bterms$fdpars$disc$value == 1) &&
+    !has_cs(bterms)
+}
+
 # prepare distribution and arguments for use in Stan
 sdist <- function(dist, ..., shift = "") {
   args <- sargs(...)
