@@ -32,8 +32,10 @@ A <- ape::vcv.phylo(phylo)
 
 ## ---- results='hide'--------------------------------------------------------------------
 model_simple <- brm(
-  phen ~ cofactor + (1|phylo), data = data_simple, 
-  family = gaussian(), cov_ranef = list(phylo = A),
+  phen ~ cofactor + (1|gr(phylo, cov = A)), 
+  data = data_simple, 
+  family = gaussian(), 
+  data2 = list(A = A),
   prior = c(
     prior(normal(0, 10), "b"),
     prior(normal(0, 50), "Intercept"),
@@ -63,9 +65,10 @@ head(data_repeat)
 
 ## ---- results='hide'--------------------------------------------------------------------
 model_repeat1 <- brm(
-  phen ~ spec_mean_cf + (1|phylo) + (1|species), 
-  data = data_repeat, family = gaussian(), 
-  cov_ranef = list(phylo = A),
+  phen ~ spec_mean_cf + (1|gr(phylo, cov = A)) + (1|species), 
+  data = data_repeat, 
+  family = gaussian(), 
+  data2 = list(A = A),
   prior = c(
     prior(normal(0,10), "b"),
     prior(normal(0,50), "Intercept"),
@@ -117,9 +120,9 @@ head(data_fisher)
 
 ## ---- results='hide'--------------------------------------------------------------------
 model_fisher <- brm(
-  Zr | se(sqrt(1 / (N - 3))) ~ 1 + (1|phylo) + (1|obs), 
+  Zr | se(sqrt(1 / (N - 3))) ~ 1 + (1|gr(phylo, cov = A)) + (1|obs), 
   data = data_fisher, family = gaussian(), 
-  cov_ranef = list(phylo = A),
+  data2 = list(A = A),
   prior = c(
     prior(normal(0, 10), "Intercept"),
     prior(student_t(3, 0, 10), "sd")
@@ -142,9 +145,9 @@ head(data_pois)
 
 ## ---- results='hide'--------------------------------------------------------------------
 model_pois <- brm(
-  phen_pois ~ cofactor + (1|phylo) + (1|obs), 
+  phen_pois ~ cofactor + (1|gr(phylo, cov = A)) + (1|obs), 
   data = data_pois, family = poisson("log"), 
-  cov_ranef = list(phylo = A),
+  data2 = list(A = A),
   chains = 2, cores = 2, iter = 4000,
   control = list(adapt_delta = 0.95)
 )
@@ -155,9 +158,9 @@ plot(conditional_effects(model_pois), points = TRUE)
 
 ## ---- results='hide'--------------------------------------------------------------------
 model_normal <- brm(
-  phen_pois ~ cofactor + (1|phylo), 
+  phen_pois ~ cofactor + (1|gr(phylo, cov = A)), 
   data = data_pois, family = gaussian(), 
-  cov_ranef = list(phylo = A),
+  data2 = list(A = A),
   chains = 2, cores = 2, iter = 4000,
   control = list(adapt_delta = 0.95)
 )
